@@ -24,4 +24,7 @@ feedurl fixed => (
 
 __PACKAGE__->meta->make_immutable;
 
+no Any::Moose;
+no Net::Google::DataAPI;
+
 1;
